@@ -40,22 +40,22 @@ ac_table = {
         'pod': ['--help', '--name', '--identity', '--uses', '--py-modules', '--uses-internal', '--entrypoint',
                 '--pull-latest', '--volumes', '--port-in', '--port-out', '--host-in', '--host-out', '--socket-in',
                 '--socket-out', '--port-ctrl', '--ctrl-with-ipc', '--timeout', '--timeout-ctrl', '--timeout-ready',
-                '--dump-interval', '--exit-no-dump', '--read-only', '--separated-workspace', '--pea-id',
-                '--check-version', '--compress-hwm', '--compress-lwm', '--num-part', '--role', '--skip-on-error',
-                '--memory-hwm', '--runtime', '--max-idle-time', '--daemon', '--log-config', '--log-remote',
-                '--ssh-server', '--ssh-keyfile', '--ssh-password', '--host', '--port-expose', '--port-grpc',
-                '--max-message-size', '--proxy', '--parallel', '--shards', '--polling', '--scheduling', '--uses-before',
-                '--uses-after', '--shutdown-idle'],
+                '--dump-interval', '--exit-no-dump', '--read-only', '--separated-workspace', '--pea-id', '--num-part',
+                '--role', '--skip-on-error', '--memory-hwm', '--runtime', '--max-idle-time', '--daemon', '--log-config',
+                '--log-remote', '--ssh-server', '--ssh-keyfile', '--ssh-password', '--host', '--port-expose',
+                '--port-grpc', '--max-message-size', '--proxy', '--parallel', '--shards', '--polling', '--scheduling',
+                '--uses-before', '--uses-after', '--shutdown-idle'],
         'flow': ['--help', '--uses', '--logserver', '--logserver-config', '--optimize-level', '--output-type',
                  '--output-path', '--inspect'],
         'gateway': ['--help', '--name', '--identity', '--uses', '--py-modules', '--uses-internal', '--entrypoint',
                     '--pull-latest', '--volumes', '--port-in', '--port-out', '--host-in', '--host-out', '--socket-in',
                     '--socket-out', '--port-ctrl', '--ctrl-with-ipc', '--timeout', '--timeout-ctrl', '--timeout-ready',
                     '--dump-interval', '--exit-no-dump', '--read-only', '--separated-workspace', '--pea-id',
-                    '--check-version', '--compress-hwm', '--compress-lwm', '--num-part', '--role', '--skip-on-error',
-                    '--memory-hwm', '--runtime', '--max-idle-time', '--daemon', '--log-config', '--log-remote',
-                    '--ssh-server', '--ssh-keyfile', '--ssh-password', '--host', '--port-expose', '--port-grpc',
-                    '--max-message-size', '--proxy', '--prefetch', '--prefetch-on-recv', '--allow-spawn', '--rest-api'],
+                    '--num-part', '--role', '--skip-on-error', '--memory-hwm', '--runtime', '--max-idle-time',
+                    '--daemon', '--log-config', '--log-remote', '--ssh-server', '--ssh-keyfile', '--ssh-password',
+                    '--host', '--port-expose', '--port-grpc', '--max-message-size', '--proxy', '--prefetch',
+                    '--prefetch-on-recv', '--allow-spawn', '--rest-api', '--check-version', '--compress',
+                    '--compress-hwm', '--compress-lwm'],
         'ping': ['--help', '--timeout', '--retries', '--print-response'],
         'check': ['--help', '--summary-exec', '--summary-driver'], 'hub login': ['--help'],
         'hub new': ['--help', '--output-dir', '--template', '--type', '--overwrite'],
@@ -70,11 +70,10 @@ ac_table = {
         'pea': ['--help', '--name', '--identity', '--uses', '--py-modules', '--uses-internal', '--entrypoint',
                 '--pull-latest', '--volumes', '--port-in', '--port-out', '--host-in', '--host-out', '--socket-in',
                 '--socket-out', '--port-ctrl', '--ctrl-with-ipc', '--timeout', '--timeout-ctrl', '--timeout-ready',
-                '--dump-interval', '--exit-no-dump', '--read-only', '--separated-workspace', '--pea-id',
-                '--check-version', '--compress-hwm', '--compress-lwm', '--num-part', '--role', '--skip-on-error',
-                '--memory-hwm', '--runtime', '--max-idle-time', '--daemon', '--log-config', '--log-remote',
-                '--ssh-server', '--ssh-keyfile', '--ssh-password', '--host', '--port-expose', '--port-grpc',
-                '--max-message-size', '--proxy'], 'log': ['--help', '--groupby-regex', '--refresh-time'],
+                '--dump-interval', '--exit-no-dump', '--read-only', '--separated-workspace', '--pea-id', '--num-part',
+                '--role', '--skip-on-error', '--memory-hwm', '--runtime', '--max-idle-time', '--daemon', '--log-config',
+                '--log-remote', '--ssh-server', '--ssh-keyfile', '--ssh-password', '--host', '--port-expose',
+                '--port-grpc', '--max-message-size', '--proxy'], 'log': ['--help', '--groupby-regex', '--refresh-time'],
         'client': ['--help', '--host', '--port-expose', '--port-grpc', '--max-message-size', '--proxy', '--batch-size',
                    '--mode', '--top-k', '--mime-type', '--callback-on-body', '--timeout-ready', '--skip-dry-run'],
         'export-api': ['--help', '--yaml-path', '--json-path']}}
