@@ -67,7 +67,7 @@ class NetworkChecker:
                     else:
                         total_success += 1
                         if args.print_response:
-                            default_logger.info(f'returns {MessageToJson(r)}')
+                            default_logger.info(f'returns {MessageToJson(r.as_pb_object)}')
                 total_time += tc.duration
                 time.sleep(1)
             if total_success < args.retries:
