@@ -232,7 +232,7 @@ class CompoundExecutor(BaseExecutor):
 
         for c in self.components:
             c.save()
-        super().save()  # do i really need to save the compound executor itself
+        super().save(filename=filename)  # do i really need to save the compound executor itself
         return True
 
     @property
@@ -337,6 +337,7 @@ class CompoundExecutor(BaseExecutor):
         if self.components:
             for c in self.components:
                 c.close()
+        super().close()
 
     @classmethod
     def to_yaml(cls, representer, data):
