@@ -2,7 +2,7 @@ __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
 import os
-from typing import Tuple, Union, List
+from typing import Tuple, Union, List, Iterator, Optional, Any
 
 import numpy as np
 
@@ -12,7 +12,7 @@ from ...helper import call_obj_fn, cached_property, get_readable_size
 
 
 class BaseIndexer(BaseExecutor):
-    """``BaseIndexer`` stores and searches with vectors.
+    """ base class for storing and searching any kind of data structure
 
     The key functions here are :func:`add` and :func:`query`.
     One can decorate them with :func:`jina.decorator.require_train`,
@@ -53,12 +53,7 @@ class BaseIndexer(BaseExecutor):
         self.handler_mutex = True  #: only one handler at a time by default
         self._size = 0
 
-    def add(self, keys: 'np.ndarray', vectors: 'np.ndarray', *args, **kwargs):
-        """Add new chunks and their vector representations
-
-        :param keys: ``chunk_id`` in 1D-ndarray, shape B x 1
-        :param vectors: vector representations in B x D
-        """
+    def add(self, *args, **kwargs):
         raise NotImplementedError
 
     def post_init(self):
@@ -66,14 +61,7 @@ class BaseIndexer(BaseExecutor):
         self.index_filename = self.index_filename or self.name
         self.is_handler_loaded = False
 
-    def query(self, keys: 'np.ndarray', top_k: int, *args, **kwargs) -> Tuple['np.ndarray', 'np.ndarray']:
-        """Find k-NN using query vectors, return chunk ids and chunk scores
-
-        :param keys: query vectors in ndarray, shape B x D
-        :param top_k: int, the number of nearest neighbour to return
-        :return: a tuple of two ndarray.
-            The first is ids in shape B x K (`dtype=int`), the second is scores in shape B x K (`dtype=float`)
-        """
+    def query(self, *args, **kwargs):
         raise NotImplementedError
 
     @property
@@ -90,6 +78,7 @@ class BaseIndexer(BaseExecutor):
         .. note::
             :attr:`query_handler` and :attr:`write_handler` are by default mutex
         """
+        r = None
         if (not self.handler_mutex or not self.is_handler_loaded) and self.is_exist:
             r = self.get_query_handler()
             if r is None:
@@ -99,7 +88,14 @@ class BaseIndexer(BaseExecutor):
             else:
                 self.logger.info(f'indexer size: {self.size}')
                 self.is_handler_loaded = True
-            return r
+        if r is None:
+            r = self.null_query_handler
+        return r
+
+    @cached_property
+    def null_query_handler(self) -> Optional[Any]:
+        """The empty query handler when :meth:`get_query_handler` fails"""
+        return
 
     @property
     def is_exist(self) -> bool:
@@ -181,6 +177,24 @@ class BaseVectorIndexer(BaseIndexer):
         """
         raise NotImplementedError
 
+    def add(self, keys: 'np.ndarray', vectors: 'np.ndarray', *args, **kwargs):
+        """Add new chunks and their vector representations
+
+        :param keys: ``chunk_id`` in 1D-ndarray, shape B x 1
+        :param vectors: vector representations in B x D
+        """
+        raise NotImplementedError
+
+    def query(self, keys: 'np.ndarray', top_k: int, *args, **kwargs) -> Tuple['np.ndarray', 'np.ndarray']:
+        """Find k-NN using query vectors, return chunk ids and chunk scores
+
+        :param keys: query vectors in ndarray, shape B x D
+        :param top_k: int, the number of nearest neighbour to return
+        :return: a tuple of two ndarray.
+            The first is ids in shape B x K (`dtype=int`), the second is scores in shape B x K (`dtype=float`)
+        """
+        raise NotImplementedError
+
 
 class BaseKVIndexer(BaseIndexer):
     """An abstract class for key-value indexer.
@@ -190,6 +204,24 @@ class BaseKVIndexer(BaseIndexer):
     It can be used to tell whether an indexer is key-value indexer, via ``isinstance(a, BaseKVIndexer)``
     """
 
+    def add(self, keys: Iterator[int], values: Iterator[bytes], *args, **kwargs):
+        raise NotImplementedError
+
+    def query(self, key: Any) -> Optional[Any]:
+        """ Find the protobuf chunk/doc using id
+
+        :param key: ``id``
+        :return: protobuf chunk or protobuf document
+        """
+        raise NotImplementedError
+
+    def __getitem__(self, key: Any) -> Optional[Any]:
+        return self.query(key)
+
+
+class UniqueVectorIndexer(CompoundExecutor):
+    """A frequently used pattern for combining a :class:`BaseVectorIndexer` and a :class:`DocIDCache` """
+
 
 class CompoundIndexer(CompoundExecutor):
     """A Frequently used pattern for combining A :class:`BaseVectorIndexer` and :class:`BaseKVIndexer`.
@@ -217,7 +249,7 @@ class CompoundIndexer(CompoundExecutor):
             metas:
               name: vecidx  # a customized name
               workspace: $TEST_WORKDIR
-          - !BasePbIndexer
+          - !BinaryPbIndexer
             with:
               index_filename: chunk.gz
             metas:
