@@ -7,7 +7,7 @@ from typing import Iterator, Optional
 import numpy as np
 
 from . import BaseKVIndexer
-from ...proto import jina_pb2
+from ..compound import CompoundExecutor
 
 
 class BinaryPbIndexer(BaseKVIndexer):
@@ -27,7 +27,7 @@ class BinaryPbIndexer(BaseKVIndexer):
     class ReadHandler:
         def __init__(self, path):
             with open(path + '.head', 'rb') as fp:
-                tmp = np.frombuffer(fp.read(), dtype=np.uint64).reshape([-1, 4])
+                tmp = np.frombuffer(fp.read(), dtype=np.int64).reshape([-1, 4])
                 self.header = {r[0]: r[1:] for r in tmp}
             self._body = open(path, 'r+b')
             self.body = self._body.fileno()
@@ -52,31 +52,34 @@ class BinaryPbIndexer(BaseKVIndexer):
         self._start = 0
         self._page_size = mmap.ALLOCATIONGRANULARITY
 
-    def add(self, docs: Iterator['jina_pb2.Document'], *args, **kwargs):
-        for d in docs:
-            s = d.SerializeToString()
-            l = len(s)  #: the length
+    def add(self, keys: Iterator[int], values: Iterator[bytes], *args, **kwargs):
+        for key, value in zip(keys, values):
+            l = len(value)  #: the length
             p = int(self._start / self._page_size) * self._page_size  #: offset of the page
-            r = self._start % self._page_size  #: the reminder, i.e. the start position given the offset
-            self.write_handler.header.write(np.array((d.id, p, r, r + l), dtype=np.uint64).tobytes())
+            r = self._start % self._page_size  #: the remainder, i.e. the start position given the offset
+            self.write_handler.header.write(
+                np.array(
+                    (key, p, r, r + l),
+                    dtype=np.int64
+                ).tobytes()
+            )
             self._start += l
-            self.write_handler.body.write(s)
+            self.write_handler.body.write(value)
             self._size += 1
             # print(f'l: {l} p: {p} r: {r} r+l: {r + l} size: {self._size}')
 
-    def query(self, key: int) -> Optional['jina_pb2.Document']:
-        # print(f'key={key}')
+    def query(self, key: int) -> Optional[bytes]:
         pos_info = self.query_handler.header.get(key, None)
         if pos_info is not None:
             p, r, l = pos_info
             with mmap.mmap(self.query_handler.body, offset=p, length=l) as m:
-                # print(f'{p}\t{r}\t{l}')
-                b = jina_pb2.Document()
-                b.ParseFromString(m[r:])
-                # print(b)
-                return b
+                return m[r:]
 
 
 class DataURIPbIndexer(BinaryPbIndexer):
     """Shortcut for :class:`DocPbIndexer` equipped with ``requests.on`` for storing doc-level protobuf and data uri info,
     differ with :class:`ChunkPbIndexer` only in ``requests.on`` """
+
+
+class UniquePbIndexer(CompoundExecutor):
+    """A frequently used pattern for combining a :class:`BaseKVIndexer` and a :class:`DocIDCache` """
