@@ -553,7 +553,7 @@ class Flow(ExitStack):
 
     @deprecated_alias(buffer='input_fn', callback='output_fn')
     def train(self, input_fn: Union[Iterator['jina_pb2.Document'], Iterator[bytes], Callable] = None,
-              output_fn: Callable[['jina_pb2.Message'], None] = None,
+              output_fn: Callable[['jina_pb2.Request'], None] = None,
               **kwargs):
         """Do training on the current flow
 
@@ -592,7 +592,7 @@ class Flow(ExitStack):
         self._get_client(**kwargs).train(input_fn, output_fn, **kwargs)
 
     def index_ndarray(self, array: 'np.ndarray', axis: int = 0, size: int = None, shuffle: bool = False,
-                      output_fn: Callable[['jina_pb2.Message'], None] = None,
+                      output_fn: Callable[['jina_pb2.Request'], None] = None,
                       **kwargs):
         """Using numpy ndarray as the index source for the current flow
 
@@ -607,7 +607,7 @@ class Flow(ExitStack):
         self._get_client(**kwargs).index(input_numpy(array, axis, size, shuffle), output_fn, **kwargs)
 
     def search_ndarray(self, array: 'np.ndarray', axis: int = 0, size: int = None, shuffle: bool = False,
-                       output_fn: Callable[['jina_pb2.Message'], None] = None,
+                       output_fn: Callable[['jina_pb2.Request'], None] = None,
                        **kwargs):
         """Use a numpy ndarray as the query source for searching on the current flow
 
@@ -623,7 +623,7 @@ class Flow(ExitStack):
 
     def index_lines(self, lines: Iterator[str] = None, filepath: str = None, size: int = None,
                     sampling_rate: float = None, read_mode='r',
-                    output_fn: Callable[['jina_pb2.Message'], None] = None,
+                    output_fn: Callable[['jina_pb2.Request'], None] = None,
                     **kwargs):
         """ Use a list of lines as the index source for indexing on the current flow
 
@@ -642,7 +642,7 @@ class Flow(ExitStack):
 
     def index_files(self, patterns: Union[str, List[str]], recursive: bool = True,
                     size: int = None, sampling_rate: float = None, read_mode: str = None,
-                    output_fn: Callable[['jina_pb2.Message'], None] = None,
+                    output_fn: Callable[['jina_pb2.Request'], None] = None,
                     **kwargs):
         """ Use a set of files as the index source for indexing on the current flow
 
@@ -662,7 +662,7 @@ class Flow(ExitStack):
 
     def search_files(self, patterns: Union[str, List[str]], recursive: bool = True,
                      size: int = None, sampling_rate: float = None, read_mode: str = None,
-                     output_fn: Callable[['jina_pb2.Message'], None] = None,
+                     output_fn: Callable[['jina_pb2.Request'], None] = None,
                      **kwargs):
         """ Use a set of files as the query source for searching on the current flow
 
@@ -682,7 +682,7 @@ class Flow(ExitStack):
 
     def search_lines(self, filepath: str = None, lines: Iterator[str] = None, size: int = None,
                      sampling_rate: float = None, read_mode='r',
-                     output_fn: Callable[['jina_pb2.Message'], None] = None,
+                     output_fn: Callable[['jina_pb2.Request'], None] = None,
                      **kwargs):
         """ Use a list of files as the query source for searching on the current flow
 
@@ -701,7 +701,7 @@ class Flow(ExitStack):
 
     @deprecated_alias(buffer='input_fn', callback='output_fn')
     def index(self, input_fn: Union[Iterator[Union['jina_pb2.Document', bytes]], Callable] = None,
-              output_fn: Callable[['jina_pb2.Message'], None] = None,
+              output_fn: Callable[['jina_pb2.Request'], None] = None,
               **kwargs):
         """Do indexing on the current flow
 
@@ -741,7 +741,7 @@ class Flow(ExitStack):
 
     @deprecated_alias(buffer='input_fn', callback='output_fn')
     def search(self, input_fn: Union[Iterator[Union['jina_pb2.Document', bytes]], Callable] = None,
-               output_fn: Callable[['jina_pb2.Message'], None] = None,
+               output_fn: Callable[['jina_pb2.Request'], None] = None,
                **kwargs):
         """Do searching on the current flow
 
