@@ -9,4 +9,4 @@ class MaxRanker(Chunk2DocRanker):
     """
 
     def _get_score(self, match_idx, query_chunk_meta, match_chunk_meta, *args, **kwargs):
-        return self.get_doc_id(match_idx), match_idx[:, self.col_score].max()
+        return self.get_doc_id(match_idx), match_idx[self.COL_SCORE].max()
