@@ -307,17 +307,6 @@ def set_pea_parser(parser=None):
                      if _SHOW_ALL_ARGS else argparse.SUPPRESS)
 
     gp5 = add_arg_group(parser, 'pea messaging arguments')
-    gp5.add_argument('--check-version', action='store_true', default=False,
-                     help='comparing the jina and proto version of incoming message with local setup, '
-                          'mismatch raise an exception')
-    gp5.add_argument('--compress-hwm', type=int, default=-1,
-                     help='the high watermark that triggers the message compression. '
-                          'message bigger than this HWM (in bytes) will be compressed by lz4 algorithm.'
-                          'set this to -1 to disable this feature.')
-    gp5.add_argument('--compress-lwm', type=float, default=1.,
-                     help='the low watermark that enables the sending of a compressed message. '
-                          'compression rate (after_size/before_size) lower than this LWM will be considered as successeful '
-                          'compression, and will be sent. Otherwise, it will send the original message without compression')
     gp5.add_argument('--num-part', type=int, default=0,
                      help='the number of replicated message sent to the next Pod, 0 and 1 means single part'
                      if _SHOW_ALL_ARGS else argparse.SUPPRESS)
@@ -478,7 +467,7 @@ def _set_grpc_parser(parser=None):
 
 
 def set_gateway_parser(parser=None):
-    from .enums import SocketType
+    from .enums import SocketType, CompressAlgo
     if not parser:
         parser = set_base_parser()
     set_pea_parser(parser)
@@ -498,6 +487,23 @@ def set_gateway_parser(parser=None):
     gp1.add_argument('--rest-api', action='store_true', default=False,
                      help='use REST-API as the interface instead of gRPC with port number '
                           'set to the value of "port-expose"')
+
+    gp2 = add_arg_group(parser, 'envelope attribute arguments')
+    gp2.add_argument('--check-version', action='store_true', default=False,
+                     help='comparing the jina and proto version of incoming message with local setup, '
+                          'mismatch raise an exception')
+    gp2.add_argument('--compress', choices=list(CompressAlgo), type=CompressAlgo.from_string,
+                     default=CompressAlgo.LZ4,
+                     help='the algorithm used for compressing request data, this can reduce the network overhead but may '
+                          'increase CPU usage')
+    gp2.add_argument('--compress-hwm', type=int, default=100,
+                     help='the high watermark that triggers the message compression. '
+                          'message bigger than this HWM (in bytes) will be compressed by lz4 algorithm.'
+                          'set this to 0 to disable this feature.')
+    gp2.add_argument('--compress-lwm', type=float, default=0.9,
+                     help='the low watermark that enables the sending of a compressed message. '
+                          'compression rate (after_size/before_size) lower than this LWM will be considered as successeful '
+                          'compression, and will be sent. Otherwise, it will send the original message without compression')
     # gp1.add_argument('--to-datauri', action='store_true', default=False,
     #                  help='always represent the result document with data URI, instead of using buffer/blob/text')
     return parser
